/**
 * @file
 * Color widget synchronization and validation functionality.
 */

(function ($, Drupal, once) {
  Drupal.behaviors.wipColorWidgetSync = {
    attach(context) {
      /**
       * Normalize hex value by removing # and whitespace
       *
       * @param  {string} value - The hex value to normalize.
       * @return {string} The normalized hex value without # and whitespace.
       */
      function normalizeHexValue(value) {
        if (!value) {
          return '';
        }
        return value.toString().replace(/[#\s]/g, '');
      }

      /**
       * Validate if a string is a valid hex color
       *
       * @param  {string} hex - The hex color string to validate.
       * @return {boolean} True if valid hex color, false otherwise.
       */
      function validateHexColor(hex) {
        if (!hex) {
          return false;
        }

        // Check if it's a valid hex pattern (3 or 6 characters)
        const hexPattern3 = /^[0-9A-Fa-f]{3}$/;
        const hexPattern6 = /^[0-9A-Fa-f]{6}$/;

        return hexPattern3.test(hex) || hexPattern6.test(hex);
      }

      /**
       * Ensure hex value is 6 characters (expand 3-char to 6-char)
       *
       * @param  {string} hex - The hex color string to normalize to 6 characters.
       * @return {string} The 6-character hex color string.
       */
      function ensureFullHex(hex) {
        if (!hex) {
          return '000000';
        }

        // If 3 characters, expand to 6
        if (hex.length === 3) {
          return hex
            .split('')
            .map((char) => char + char)
            .join('');
        }

        // If 6 characters, return as is
        if (hex.length === 6) {
          return hex;
        }

        // If less than 3, pad with zeros
        if (hex.length < 3) {
          return hex.padEnd(6, '0');
        }

        // If between 3 and 6, truncate to 6
        if (hex.length > 6) {
          return hex.substring(0, 6);
        }

        return hex.padEnd(6, '0');
      }

      /**
       * Clear validation error styling
       *
       * @param {jQuery} $container - The jQuery object representing the color input container.
       */
      function clearValidationError($container) {
        $container.removeClass('is-invalid');
        const containerId =
          $container.attr('id') ||
          $container.find('input').attr('id') ||
          'unknown';
        const errorId = `error-${containerId}`;

        $(`#${errorId}`).remove();
      }

      /**
       * Add validation error styling
       *
       * @param {jQuery} $container - The jQuery object representing the color input container.
       */
      function addValidationError($container) {
        $container.addClass('is-invalid');

        const $parentContainer = $container.closest('.wip-color-group');
        const containerId =
          $container.attr('id') ||
          $container.find('input').attr('id') ||
          'unknown';
        const errorId = `error-${containerId}`;
        // Add error message if not already present
        if (!$parentContainer.find(`#${errorId}`).length) {
          const errorMessage =
            `<div id="${
              errorId
            }" class="wip-color-error form-item__error-message" style="` +
            `color: #dc3545; ` +
            `font-size: 0.875rem; ` +
            `margin-top: 0.25rem; ` +
            `margin-bottom: 0.5rem; ` +
            `padding: 0.15rem 0; ` +
            `line-height: 1.2; ` +
            `position: relative; ` +
            `z-index: 10;` +
            `">${Drupal.t(
              'Please enter a valid hex color (e.g., #FF0000 or #F00)',
            )}</div>`;

          // Inserta el mensaje después del contenedor que tiene overflow: hidden
          $container.after(errorMessage);
        }
      }

      /**
       * Setup synchronization between color picker and hex input
       *
       * @param {jQuery} $colorPicker - The jQuery object for the color picker input.
       * @param {jQuery} $hexInput - The jQuery object for the hex input field.
       */
      function setupColorSync($colorPicker, $hexInput) {
        const $container = $colorPicker.closest('.wip-color-input-wrapper');

        // Sync color picker to hex input
        $colorPicker.on('input change', function () {
          const colorValue = $(this).val();
          const hexValue = colorValue.replace('#', '').toUpperCase();
          $hexInput.val(hexValue);
          clearValidationError($container);
        });

        // Sync hex input to color picker
        $hexInput.on('input', function () {
          const hexValue = $(this).val();
          const normalizedHex = normalizeHexValue(hexValue);

          if (validateHexColor(normalizedHex)) {
            const fullHex = ensureFullHex(normalizedHex);
            $colorPicker.val(`#${fullHex}`);
            $(this).val(fullHex.toUpperCase());
            clearValidationError($container);
          } else {
            addValidationError($container);
          }
        });

        // Validate on blur to ensure final value is correct
        $hexInput.on('blur', function () {
          const hexValue = $(this).val();
          const normalizedHex = normalizeHexValue(hexValue);

          if (hexValue && !validateHexColor(normalizedHex)) {
            addValidationError($container);
          } else if (hexValue && validateHexColor(normalizedHex)) {
            const fullHex = ensureFullHex(normalizedHex);
            $colorPicker.val(`#${fullHex}`);
            $(this).val(fullHex.toUpperCase());
            clearValidationError($container);
          }
        });

        // Initialize with current values
        const initialColorValue = $colorPicker.val();
        const initialHexValue = $hexInput.val();

        if (initialColorValue && initialColorValue !== '#000000') {
          const hexValue = initialColorValue.replace('#', '').toUpperCase();
          $hexInput.val(hexValue);
        } else if (initialHexValue) {
          const normalizedHex = normalizeHexValue(initialHexValue);
          if (validateHexColor(normalizedHex)) {
            const fullHex = ensureFullHex(normalizedHex);
            $colorPicker.val(`#${fullHex}`);
            $hexInput.val(fullHex.toUpperCase());
          }
        }
      }

      /**
       * Initialize color synchronization for a widget
       *
       * @param {jQuery} $widget - The jQuery object representing the widget container.
       */
      function initializeColorSync($widget) {
        // Find all color picker and hex input pairs
        const colorPickers = $widget.find('.wip-color-picker');

        colorPickers.each(function () {
          const $colorPicker = $(this);
          const targetHexId = $colorPicker.data('target-hex');
          const $hexInput = $widget.find(`#${targetHexId}`);

          if ($hexInput.length) {
            setupColorSync($colorPicker, $hexInput);
          }
        });
      }

      /**
       * Check if there are validation errors in the form
       *
       * @param  {jQuery} $form - The jQuery object representing the form to check.
       * @return {boolean} True if there are validation errors, false otherwise.
       */
      function checkForValidationErrors($form) {
        const $invalidContainers = $form.find(
          '.wip-color-input-wrapper.is-invalid',
        );
        return $invalidContainers.length > 0;
      }

      /**
       * Focus on the first invalid color field
       *
       * @param {jQuery} $form - The jQuery object representing the form to search for invalid fields.
       */
      function focusFirstInvalidField($form) {
        const $firstInvalid = $form
          .find('.wip-color-input-wrapper.is-invalid')
          .first();
        if ($firstInvalid.length) {
          const $hexInput = $firstInvalid.find('.wip-hex-input');
          if ($hexInput.length) {
            setTimeout(function () {
              $hexInput.focus().select();
            }, 400);
          }
        }
      }

      /**
       * Show a general form error message
       *
       * @param {jQuery} $form - The jQuery object representing the form to show the error message in.
       */
      function showFormErrorMessage($form) {
        // Remove any existing error message
        $form.find('.wip-color-form-error').remove();

        // Create error message
        const errorMessage =
          `<div class="wip-color-form-error messages messages--error" style="` +
          `background-color: #fcf4f4; ` +
          `border: 1px solid #e7c3c3; ` +
          `color: #a51b00; ` +
          `padding: 10px 15px; ` +
          `margin: 10px 0; ` +
          `border-radius: 4px; ` +
          `position: relative; ` +
          `z-index: 1000;` +
          `">` +
          `<strong>${Drupal.t('Error')}:</strong> ${Drupal.t(
            'Please correct the color field errors before submitting the form.',
          )}</div>`;

        // Insert at the top of the form
        $form.prepend(errorMessage);

        // Scroll to the error message
        $('html, body').animate(
          {
            scrollTop: $form.find('.wip-color-form-error').offset().top - 50,
          },
          300,
        );
      }
      /**
       * Initialize form validation to prevent submission with errors
       *
       * @param {jQuery} $widget - The jQuery object representing the widget container.
       */
      function initializeFormValidation($widget) {
        // Find the form that contains this widget
        const $form = $widget.closest('form');

        if ($form.length) {
          // Add form validation handler
          $form.on('submit.wipColorValidation', function (e) {
            // Check if there are any validation errors in color widgets
            const hasErrors = checkForValidationErrors($form);

            if (hasErrors) {
              e.preventDefault();
              e.stopImmediatePropagation();

              // Show a general error message
              showFormErrorMessage($form);

              // Focus on the first invalid field
              focusFirstInvalidField($form);

              return false;
            }
          });
        }
      }

      const containers = once(
        'wip-color-ux',
        '.wip-color-widget-container',
        context,
      );

      containers.forEach(function initializeWidget(widget) {
        const $widget = $(widget);
        const $radioButtons = $widget.find('.wip-color-type-selector');
        const $solidContainer = $widget.find('.wip-color-solid-container');
        const $gradientContainer = $widget.find(
          '.wip-color-gradient-container',
        );

        // Initialize color synchronization for all color inputs in this widget
        initializeColorSync($widget);

        // Initialize form validation
        initializeFormValidation($widget);

        function updateContainerVisibility($selectedRadio) {
          const selectedType = $selectedRadio.val();
          $solidContainer.removeClass('is-visible');
          $gradientContainer.removeClass('is-visible');
          setTimeout(function showSelectedContainer() {
            if (selectedType === 'solid') {
              $solidContainer.addClass('is-visible');
            } else if (selectedType === 'gradient') {
              $gradientContainer.addClass('is-visible');
            }
          }, 50);
        }

        $radioButtons.on('change', function handleRadioChange() {
          updateContainerVisibility($(this));
        });

        setTimeout(function initializeSelectedState() {
          const $initialSelected = $radioButtons.filter(':checked');
          if ($initialSelected.length) {
            updateContainerVisibility($initialSelected);
          }
        }, 100);
      });
    },
  };
})(jQuery, Drupal, once);
