/**
 * @file
 * JavaScript behavior for WIP Range widget.
 */

(function wipRangeWidgetModule($, Drupal, drupalSettings) {
  /**
   * WIP Range Widget behavior.
   */
  Drupal.behaviors.wipRangeWidget = {
    attach(context, settings) {
      const instances = settings.wipRangeWidget?.instances || {};

      Object.keys(instances).forEach(
        function wipRangeWidgetInstanceHandler(instanceId) {
          const config = instances[instanceId];
          const wrapper = context.querySelector(
            `[data-wip-range-widget-id="${instanceId}"]`,
          );

          if (!wrapper || wrapper.dataset.wipInitialized) {
            return;
          }

          wrapper.dataset.wipInitialized = 'true';

          const rangeInput = wrapper.querySelector(`#${config.rangeId}`);
          const unitSelect = wrapper.querySelector(`#${config.unitId}`);
          const valueHidden = wrapper.querySelector(
            `[data-wip-range-value="${instanceId}"]`,
          );
          const unitHidden = wrapper.querySelector(
            `[data-wip-range-unit="${instanceId}"]`,
          );
          const display = config.displayId
            ? wrapper.querySelector(`#${config.displayId}`)
            : null;
          const minDisplay = wrapper.querySelector(
            `[data-min-display="${config.rangeId}"]`,
          );
          const maxDisplay = wrapper.querySelector(
            `[data-max-display="${config.rangeId}"]`,
          );

          if (!rangeInput || !unitSelect) {
            return;
          }

          function formatValue(value, unit) {
            const numValue = parseFloat(value);
            const unitConfig = config.unitRanges[unit] || config.fallbackConfig;

            const step = unitConfig.step || 1;
            const decimals = step < 1 ? 2 : 0;

            const formatted =
              decimals > 0
                ? numValue.toFixed(decimals).replace(/\.?0+$/, '')
                : Math.round(numValue).toString();

            return formatted + unit;
          }

          function updateValue() {
            const { value } = rangeInput;
            const unit = unitSelect.value;
            const formattedValue = formatValue(value, unit);

            if (display) {
              display.textContent = formattedValue;
            }
            if (valueHidden) {
              valueHidden.value = value;
            }
            if (unitHidden) {
              unitHidden.value = unit;
            }

            const event = new CustomEvent('wipRangeChange', {
              detail: {
                instanceId,
                value: parseFloat(value),
                unit,
                formatted: formattedValue,
                element: wrapper,
              },
            });
            wrapper.dispatchEvent(event);
          }

          function updateRangeConfig(unit) {
            const unitConfig = config.unitRanges[unit] || config.fallbackConfig;

            // Update range input attributes
            rangeInput.min = unitConfig.min;
            rangeInput.max = unitConfig.max;
            rangeInput.step = unitConfig.step;

            // Clamp current value to new range
            const currentValue = parseFloat(rangeInput.value) || unitConfig.min;
            const clampedValue = Math.max(
              unitConfig.min,
              Math.min(unitConfig.max, currentValue),
            );

            if (clampedValue !== currentValue) {
              rangeInput.value = clampedValue;
              updateValue();
            }

            // Update min/max displays
            if (minDisplay) {
              minDisplay.textContent = unitConfig.min;
            }
            if (maxDisplay) {
              maxDisplay.textContent = unitConfig.max;
            }
          }

          function initialize() {
            // Set initial range configuration
            updateRangeConfig(unitSelect.value);

            // Set initial display
            updateValue();

            // Add event listeners
            rangeInput.addEventListener('input', updateValue);
            rangeInput.addEventListener('change', updateValue);

            unitSelect.addEventListener(
              'change',
              function unitSelectChangeHandler() {
                updateRangeConfig(this.value);
                updateValue();
              },
            );

            // Add keyboard support for range input
            rangeInput.addEventListener(
              'keydown',
              function rangeInputKeydownHandler(e) {
                const step = parseFloat(this.step) || 1;
                const currentValue = parseFloat(this.value) || 0;
                let newValue = currentValue;

                switch (e.key) {
                  case 'ArrowUp':
                  case 'ArrowRight':
                    newValue = Math.min(
                      parseFloat(this.max),
                      currentValue + step,
                    );
                    break;
                  case 'ArrowDown':
                  case 'ArrowLeft':
                    newValue = Math.max(
                      parseFloat(this.min),
                      currentValue - step,
                    );
                    break;
                  case 'Home':
                    newValue = parseFloat(this.min);
                    break;
                  case 'End':
                    newValue = parseFloat(this.max);
                    break;
                  default:
                    return; // Don't prevent default for other keys
                }

                if (newValue !== currentValue) {
                  this.value = newValue;
                  updateValue();
                  e.preventDefault();
                }
              },
            );
          }

          // Initialize the widget
          initialize();
        },
      );
    },

    detach(context, settings, trigger) {
      if (trigger === 'unload') {
        // Clean up any event listeners or resources if needed
        const instances = settings.wipRangeWidget?.instances || {};

        Object.keys(instances).forEach(
          function wipRangeWidgetDetachHandler(instanceId) {
            const wrapper = context.querySelector(
              `[data-wip-range-widget-id="${instanceId}"]`,
            );
            if (wrapper) {
              delete wrapper.dataset.wipInitialized;
            }
          },
        );
      }
    },
  };

  /**
   * Utility function to get WIP Range widget values programmatically.
   */
  Drupal.wipRange = {
    /**
     * Get value from a WIP Range widget.
     *
     * @param  {string} instanceId - The widget instance ID
     * @return {Object|null} Object with value, unit, and formatted properties
     */
    getValue(instanceId) {
      const wrapper = document.querySelector(
        `[data-wip-range-widget-id="${instanceId}"]`,
      );
      if (!wrapper) {
        return null;
      }

      const valueInput = wrapper.querySelector(
        `[data-wip-range-value="${instanceId}"]`,
      );
      const unitInput = wrapper.querySelector(
        `[data-wip-range-unit="${instanceId}"]`,
      );

      if (!valueInput || !unitInput) {
        return null;
      }

      const value = parseFloat(valueInput.value);
      const unit = unitInput.value;

      return {
        value,
        unit,
        formatted: value + unit,
      };
    },

    /**
     * Set value for a WIP Range widget.
     *
     * @param  {string} instanceId - The widget instance ID
     * @param  {number} value - The numeric value
     * @param  {string} unit - The unit (optional)
     * @return {boolean} True if the value was set successfully, false otherwise
     */
    setValue(instanceId, value, unit) {
      const wrapper = document.querySelector(
        `[data-wip-range-widget-id="${instanceId}"]`,
      );
      if (!wrapper) {
        return false;
      }

      const config = drupalSettings.wipRangeWidget?.instances?.[instanceId];
      if (!config) {
        return false;
      }

      const rangeInput = wrapper.querySelector(`#${config.rangeId}`);
      const unitSelect = wrapper.querySelector(`#${config.unitId}`);

      if (!rangeInput) {
        return false;
      }

      // Set unit first if provided
      if (unit && unitSelect) {
        unitSelect.value = unit;
        unitSelect.dispatchEvent(new Event('change'));
      }

      // Set value
      rangeInput.value = value;
      rangeInput.dispatchEvent(new Event('input'));

      return true;
    },
  };
})(jQuery, Drupal, drupalSettings);
