<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;
use Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipRangeItem;

/**
 * Plugin implementation of the 'wip_range_formatter' formatter.
 *
 * @extends FormatterBase<FieldItemListInterface<WipRangeItem>>
 *
 * @FieldFormatter(
 *   id = "wip_range_formatter",
 *   label = @Translation("WIP Range Default"),
 *   field_types = {"wip_range"}
 * )
 */
final class WipRangeFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   *
   * @return array<string, mixed>
   *   An array of default settings for the formatter.
   */
  public static function defaultSettings(): array {
    return [
      'format' => 'value_unit',
      'decimal_places' => 2,
      'separator' => '',
      'css_property' => FALSE,
      'css_property_name' => 'width',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form elements for the formatter settings.
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $form['format'] = [
      '#type' => 'radios',
      '#title' => $this->t('Display format'),
      '#options' => [
        'value_only' => $this->t('Value only (e.g., "320")'),
        'value_unit' => $this->t('Value with unit (e.g., "320px")'),
        'unit_only' => $this->t('Unit only (e.g., "px")'),
      ],
      '#default_value' => $settings['format'],
    ];

    $form['decimal_places'] = [
      '#type' => 'number',
      '#title' => $this->t('Decimal places'),
      '#description' => $this->t('Number of decimal places to display for float values. Set to 0 to display as integer.'),
      '#default_value' => $settings['decimal_places'],
      '#min' => 0,
      '#max' => 6,
      '#states' => [
        'visible' => [
          ':input[name$="[format]"]' => [
        ['value' => 'value_only'],
        ['value' => 'value_unit'],
          ],
        ],
      ],
    ];

    $form['separator'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Separator'),
      '#description' => $this->t('Character(s) to place between value and unit. Leave empty for no separator.'),
      '#default_value' => $settings['separator'],
      '#size' => 5,
      '#states' => [
        'visible' => [
          ':input[name$="[format]"]' => ['value' => 'value_unit'],
        ],
      ],
    ];

    $form['css_property'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Wrap as CSS property'),
      '#description' => $this->t('Wrap the output in CSS property format (e.g., "width: 320px;").'),
      '#default_value' => $settings['css_property'],
    ];

    $form['css_property_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS property name'),
      '#description' => $this->t('The CSS property name to use when wrapping as CSS property.'),
      '#default_value' => $settings['css_property_name'],
      '#size' => 20,
      '#states' => [
        'visible' => [
          ':input[name$="[css_property]"]' => ['checked' => TRUE],
        ],
        'required' => [
          ':input[name$="[css_property]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();

    $format_labels = [
      'value_only' => $this->t('Value only'),
      'value_unit' => $this->t('Value with unit'),
      'unit_only' => $this->t('Unit only'),
    ];

    $summary[] = $this->t(
        'Format: @format', [
          '@format' => $format_labels[$settings['format']] ?? $settings['format'],
        ]
    );

    if (in_array($settings['format'], ['value_only', 'value_unit'])) {
      $summary[] = $this->t(
            'Decimal places: @places', [
              '@places' => $settings['decimal_places'],
            ]
        );
    }

    if ($settings['format'] === 'value_unit' && !empty($settings['separator'])) {
      $summary[] = $this->t(
            'Separator: "@separator"', [
              '@separator' => $settings['separator'],
            ]
        );
    }

    if ($settings['css_property']) {
      $summary[] = $this->t(
            'CSS property: @property', [
              '@property' => $settings['css_property_name'],
            ]
        );
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipRangeItem> $items
   *   The field items to be rendered.
   * @param string $langcode
   *   The language code.
   *
   * @return array<int, array<string, mixed>>
   *   A render array for the field.
   */
  public function viewElements(FieldItemListInterface $items, $langcode): array {
    $elements = [];
    $settings = $this->getSettings();

    foreach ($items as $delta => $item) {
      if ($item->isEmpty()) {
        continue;
      }

      /** @var \Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipRangeItem $item */
      $value = $item->get('value')->getValue();
      $unit = $item->get('unit')->getValue();

      $value_float = is_numeric($value) ? (float) $value : 0.0;
      $formatted_value = $this->formatValue($value_float, (int) $settings['decimal_places']);
      $unit = is_string($unit) ? $unit : '';

      switch ($settings['format']) {
        case 'value_only':
          $output = $formatted_value;
          break;

        case 'unit_only':
          $output = $unit;
          break;

        case 'value_unit':
        default:
          $separator = $settings['separator'] ?? '';
          $output = $formatted_value . $separator . $unit;
          break;
      }

      if ($settings['css_property'] && !empty($settings['css_property_name'])) {
        $output = $settings['css_property_name'] . ': ' . $output . ';';
        $elements[$delta] = [
          '#markup' => Markup::create($output),
        ];
      }
      else {
        $elements[$delta] = [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => $output,
          '#attributes' => [
            'class' => ['wip-range-formatted'],
            'data-value' => $value,
            'data-unit' => $unit,
            'data-formatted' => $output,
          ],
        ];

      }

    }

    return $elements;
  }

  /**
   * Format numeric value with specified decimal places.
   */
  private function formatValue(float $value, int $decimal_places): string {
    if ($decimal_places === 0) {
      return (string) (int) round($value);
    }
    return number_format($value, $decimal_places, '.', '');
  }

}
