<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Field\FieldDefinitionInterface;

/**
 * Plugin implementation of the 'wip_color' field type.
 *
 * @FieldType(
 *   id = "wip_color",
 *   label = @Translation("Wip Color"),
 *   description = @Translation("Stores color values for solid colors and gradients for WIP paragraphs."),
 *   category = @Translation("WIP"),
 *   default_widget = "wip_color_widget",
 *   default_formatter = "wip_color_formatter",
 * )
 *
 * @property string $wip_color_type
 * @property string $wip_value
 * @property string|null $wip_end_color
 * @property string|null $wip_direction
 */
final class WipColorItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   *
   * @return array<string, mixed>
   *   An array of default settings for the field type.
   */
  public static function defaultFieldSettings(): array {
    return [
      'allow_opacity' => TRUE,
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    $type = $this->get('wip_color_type')->getValue();

    if (empty($type) || $type === 'solid') {
      return empty($this->get('wip_value')->getValue());
    }

    if ($type === 'gradient') {
      return empty($this->get('wip_value')->getValue())
        || empty($this->get('wip_end_color')->getValue());
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $field_definition
   *   The field storage definition.
   *
   * @return \Drupal\Core\TypedData\DataDefinition[]
   *   An array of property data definitions.
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    $properties['wip_color_type'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Color type'))
      ->setDescription(new TranslatableMarkup("Type of color ('solid' or 'gradient')."))
      ->setRequired(TRUE);

    $properties['wip_value'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Start Color'))
      ->setDescription(new TranslatableMarkup('The hexadecimal color value (e.g., #RRGGBBAA). For solid color, this is the only value. For gradient, it is the start color.'))
      ->setRequired(FALSE);

    $properties['wip_end_color'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('End Color'))
      ->setDescription(new TranslatableMarkup('The end hexadecimal color for gradients.'))
      ->setRequired(FALSE);

    $properties['wip_direction'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Direction'))
      ->setDescription(new TranslatableMarkup('The direction for gradients (e.g., "180deg", "to right").'))
      ->setRequired(FALSE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $field_definition
   *   The field storage definition.
   *
   * @return array<string, mixed>
   *   An associative array defining the schema for the field type.
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    return [
      'columns' => [
        'wip_color_type' => [
          'type' => 'varchar',
          'length' => 10,
          'not null' => TRUE,
          'default' => 'solid',
        ],
        'wip_value' => [
          'type' => 'varchar',
          'length' => 9,
          'not null' => FALSE,
        ],
        'wip_end_color' => [
          'type' => 'varchar',
          'length' => 9,
          'not null' => FALSE,
        ],
        'wip_direction' => [
          'type' => 'varchar',
          'length' => 255,
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param \Drupal\Core\Entity\EntityInterface|null $entity
   *   The entity this field belongs to, or NULL if not available.
   * @param int $delta
   *   The delta of the field item.
   *
   * @return array<string, mixed>
   *   An array representing the default value for the field item.
   */
  public static function getDefaultValue(FieldDefinitionInterface $field_definition, $entity, $delta = 0) : array {
    $default_value = $field_definition->getDefaultValueLiteral();

    if (!empty($default_value[$delta])) {
      return $default_value[$delta];
    }

    return [];
  }

  /**
   * Applies the default value to the field item.
   */
  public function applyDefaultValue($notify = TRUE) {
    $default_value = $this->getFieldDefinition()->getDefaultValueLiteral();

    if (!empty($default_value[0])) {
      $this->setValue($default_value[0], $notify);
      return $this;
    }

    $this->setValue(
          [
            'wip_color_type' => 'solid',
            'wip_value' => NULL,
            'wip_end_color' => NULL,
            'wip_direction' => NULL,
          ], $notify
      );

    return $this;
  }

}
