<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldType;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Defines the 'wip_range' field type.
 *
 * @FieldType(
 *   id = "wip_range",
 *   label = @Translation("Wip Range"),
 *   description = @Translation("Numeric value for CSS properties like width, height, or border-width."),
 *   category = @Translation("Number"),
 *   default_widget = "wip_range_widget",
 *   default_formatter = "wip_range_formatter",
 * )
 */
final class WipRangeItem extends FieldItemBase {

  /**
   * {@inheritdoc}
   *
   * @return array<string, mixed>
   *   An array of default settings for the field type.
   */
  public static function defaultFieldSettings(): array {
    return [
      'min' => 50,
      'max' => 800,
      'step' => 1,
      'default_unit' => 'px',
      'allowed_units' => ['px', 'rem', 'em', '%', 'vw', 'vh', 'pt', 'pc'],
      'unit_ranges' => [
        'px' => ['min' => 0, 'max' => 800, 'step' => 1],
        'rem' => ['min' => 0.1, 'max' => 50.0, 'step' => 0.1],
        'em' => ['min' => 0.1, 'max' => 50.0, 'step' => 0.1],
        '%' => ['min' => 0, 'max' => 100, 'step' => 1],
        'vw' => ['min' => 1, 'max' => 100, 'step' => 1],
        'vh' => ['min' => 1, 'max' => 100, 'step' => 1],
        'pt' => ['min' => 1, 'max' => 72, 'step' => 1],
        'pc' => ['min' => 0.1, 'max' => 6.0, 'step' => 0.1],
      ],
    ] + parent::defaultFieldSettings();
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   *   The form array to which the settings elements will be added.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form elements for the field settings.
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state): array {
    $element = parent::fieldSettingsForm($form, $form_state);
    $settings = $this->getSettings();

    $element['default_unit'] = [
      '#type' => 'select',
      '#title' => $this->t('Default unit'),
      '#options' => array_combine($settings['allowed_units'], $settings['allowed_units']),
      '#default_value' => $settings['default_unit'],
      '#description' => $this->t('The default unit when the field is first loaded.'),
    ];

    $element['allowed_units'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed units'),
      '#options' => [
        'px' => $this->t('px (Pixels)'),
        'rem' => $this->t('rem (Root em)'),
        'em' => $this->t('em (Relative to font size)'),
        '%' => $this->t('% (Percentage)'),
        'vw' => $this->t('vw (Viewport width)'),
        'vh' => $this->t('vh (Viewport height)'),
        'pt' => $this->t('pt (Points)'),
        'pc' => $this->t('pc (Picas)'),
      ],
      '#default_value' => array_combine($settings['allowed_units'], $settings['allowed_units']),
      '#description' => $this->t('Select which units are available for this field.'),
    ];

    $element['min'] = [
      '#type' => 'number',
      '#title' => $this->t('Minimum value'),
      '#default_value' => $settings['min'],
      '#description' => $this->t('The minimum value for this field instance. Each field can have different min/max values.'),
      '#required' => TRUE,
      '#step' => 'any',
    ];

    $element['max'] = [
      '#type' => 'number',
      '#title' => $this->t('Maximum value'),
      '#default_value' => $settings['max'],
      '#description' => $this->t('The maximum value for this field instance. Each field can have different min/max values.'),
      '#required' => TRUE,
      '#step' => 'any',
    ];

    $element['step'] = [
      '#type' => 'number',
      '#title' => $this->t('Step value'),
      '#default_value' => $settings['step'],
      '#description' => $this->t('The step increment for the range input.'),
      '#min' => 1,
      '#required' => TRUE,
      '#step' => 'any',
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return match ($this->get('value')->getValue()) {
      NULL, '' => TRUE,
            default => FALSE,
    };
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {

    $properties['value'] = DataDefinition::create('float')
      ->setLabel(new TranslatableMarkup('Numeric value'))
      ->setDescription(new TranslatableMarkup('The numerical value from the range input.'))
      ->setRequired(TRUE);

    $properties['unit'] = DataDefinition::create('string')
      ->setLabel(new TranslatableMarkup('Unit'))
      ->setDescription(new TranslatableMarkup('The unit of measurement (px, rem, em, %, etc.).'))
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   *
   * @param \Drupal\Core\Field\FieldStorageDefinitionInterface $field_definition
   *   The field storage definition.
   *
   * @return array<string, mixed>
   *   An associative array defining the schema for the field type.
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {

    return [
      'columns' => [
        'value' => [
          'type' => 'numeric',
          'precision' => 10,
          'scale' => 2,
          'not null' => FALSE,
        ],
        'unit' => [
          'type' => 'varchar',
          'length' => 10,
          'not null' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @return array{value: float, unit: string}
   *   Generates a sample value for the field type.
   */
  public static function generateSampleValue(FieldDefinitionInterface $field_definition): array {
    $settings = $field_definition->getSettings();
    $allowed_units = array_filter($settings['allowed_units']);
    $unit = $allowed_units ? array_rand($allowed_units) : 'px';
    $unit_ranges = $settings['unit_ranges'][$unit] ?? [];

    $min = (float) ($unit_ranges['min'] ?? $settings['min'] ?? 0);
    $max = (float) ($unit_ranges['max'] ?? $settings['max'] ?? 100);

    $random_value = $min + (mt_rand() / mt_getrandmax()) * ($max - $min);

    return [
      'value' => round($random_value, 2),
      'unit' => (string) $unit,
    ];
  }

  /**
   * Get the formatted value with unit.
   *
   * @return string
   *   The formatted value with unit (e.g., "320px").
   */
  public function getFormattedValue(): string {
    $value = $this->get('value')->getValue();
    $unit = $this->get('unit')->getValue();

    $value_float = is_numeric($value) ? (float) $value : 0.0;
    $unit = is_string($unit) ? $unit : '';

    if ($value === NULL || $value === '') {
      return '';
    }

    return $value_float . $unit;
  }

  /**
   * Get unit-specific range configuration.
   *
   * @param string $unit
   *   The unit to get configuration for.
   *
   * @return array<string, mixed>
   *   Array with min, max, and step values.
   */
  public function getUnitRangeConfig(string $unit): array {
    $settings = $this->getSettings();
    $unit_ranges = $settings['unit_ranges'][$unit] ?? [];

    return [
      'min' => $unit_ranges['min'] ?? $settings['min'] ?? 0,
      'max' => $unit_ranges['max'] ?? $settings['max'] ?? 100,
      'step' => $unit_ranges['step'] ?? $settings['step'] ?? 1,
    ];
  }

}
