<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;

/**
 * Defines the 'wip_color_widget' field widget.
 *
 * @FieldWidget(
 *   id = "wip_color_widget",
 *   label = @Translation("WIP Color picker"),
 *   field_types = {
 *     "wip_color"
 *   },
 *   )
 */
final class WipColorWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   *
   * Builds the form element for a single field item.
   * This method renders an HTML5 <input type="color"> element.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem> $items
   *   The field items being rendered.
   * @param int $delta
   *   The order of this item in the array of field items.
   * @param array<string, mixed> $element
   *   A form element array to populate with the widget's properties.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form element array for the widget.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {

    /** @var array{wip_color_type: string, wip_value: string, wip_end_color: string, wip_direction: string} $default_values */
    $default_values = $this->getDefaultValues($items, $delta, $form_state);

    $element += [
      '#type' => 'fieldset',
      '#attributes' => ['class' => ['wip-color-widget-container']],
    ];

    $element['#attributes'] = is_array($element['#attributes'] ?? NULL) ? $element['#attributes'] : [];
    $element['#attributes']['class'] = is_array($element['#attributes']['class'] ?? NULL)
      ? $element['#attributes']['class']
      : [];
    $element['#attributes']['class'][] = 'wip-color-widget-container';

    $element['#attached'] = is_array($element['#attached'] ?? NULL) ? $element['#attached'] : [];
    $element['#attached']['library'] = is_array($element['#attached']['library'] ?? NULL)
      ? $element['#attached']['library']
      : [];
    $element['#attached']['library'][] = 'web_initial_paragraphs/wip_color_widget_ux';

    $element['wip_color_type'] = [
      '#type' => 'radios',
      '#title' => new TranslatableMarkup('Color Type'),
      '#options' => [
        'solid' => new TranslatableMarkup('Solid'),
        'gradient' => new TranslatableMarkup('Gradient'),
      ],
      '#default_value' => $default_values['wip_color_type'],
      '#attributes' => [
        'class' => ['wip-color-type-selector'],
      ],
    ];

    $parents = array_merge(
      (array) ($element['#field_parents'] ?? []),
      [
        $this->fieldDefinition->getName(),
        $delta,
        'wip_color_type',
      ]
    );

    $selector_value = array_shift($parents);
    $selector_name = is_string($selector_value) ? $selector_value : '';
    $selector_name .= $parents ? '[' . implode('][', $parents) . ']' : '';
    $selector_path = ':input[name="' . $selector_name . '"]';

    $element['solid_container'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-solid-container']],
      '#states' => ['visible' => [$selector_path => ['value' => 'solid']]],
    ];
    $element['solid_container']['solid_color_group'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-group']],
    ];
    $element['solid_container']['solid_color_group']['color_wrapper_title'] = [
      '#markup' => '<div class="wip-color-title">' . new TranslatableMarkup('Color') . '</div>',
    ];
    $element['solid_container']['solid_color_group']['color_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-input-wrapper']],
    ];
    $element['solid_container']['solid_color_group']['color_wrapper']['wip_value'] = [
      '#type' => 'color',
      '#default_value' => $this->formatHexForInput((string) $default_values['wip_value'] ?: '#000000'),
      '#attributes' => [
        'class' => ['wip-color-picker'],
        'data-target-hex' => 'solid-hex-input-' . $delta,
      ],
    ];
    $element['solid_container']['solid_color_group']['color_wrapper']['wip_value_text'] = [
      '#type' => 'textfield',
      '#default_value' => strtoupper($this->formatHexForInput($default_values['wip_value'] ?: '#000000')),
      '#size' => 8,
      '#maxlength' => 7,
      '#attributes' => [
        'class' => ['wip-hex-input'],
        'id' => 'solid-hex-input-' . $delta,
        'data-target-color-picker' => 'wip-color-picker',
      ],
      '#field_prefix' => '#',
    ];

    $element['gradient_container'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-gradient-container']],
      '#states' => ['visible' => [$selector_path => ['value' => 'gradient']]],
    ];

    $element['gradient_container']['start_color_group'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-group']],
    ];
    $element['gradient_container']['start_color_group']['start_color_wrapper_title'] = [
      '#markup' => '<div class="wip-color-title">' . new TranslatableMarkup('Start Color') . '</div>',
    ];
    $element['gradient_container']['start_color_group']['start_color_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-input-wrapper']],
    ];
    $element['gradient_container']['start_color_group']['start_color_wrapper']['wip_value'] = [
      '#type' => 'color',
      '#default_value' => $this->formatHexForInput($default_values['wip_value'] ?: '#000000'),
      '#attributes' => [
        'class' => ['wip-color-picker'],
        'data-target-hex' => 'start-hex-input-' . $delta,
      ],
    ];
    $element['gradient_container']['start_color_group']['start_color_wrapper']['wip_value_text'] = [
      '#type' => 'textfield',
      '#default_value' => strtoupper($this->formatHexForInput($default_values['wip_value']) ?: '#000000'),
      '#size' => 8,
      '#maxlength' => 7,
      '#attributes' => [
        'class' => ['wip-hex-input'],
        'id' => 'start-hex-input-' . $delta,
        'data-target-color-picker' => 'wip-color-picker',
      ],
      '#field_prefix' => '#',
    ];

    $element['gradient_container']['end_color_group'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-group']],
    ];
    $element['gradient_container']['end_color_group']['end_color_wrapper_title'] = [
      '#markup' => '<div class="wip-color-title">' . new TranslatableMarkup('End Color') . '</div>',
    ];
    $element['gradient_container']['end_color_group']['end_color_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-color-input-wrapper']],
    ];
    $element['gradient_container']['end_color_group']['end_color_wrapper']['wip_end_color'] = [
      '#type' => 'color',
      '#default_value' => $this->formatHexForInput($default_values['wip_end_color'] ?: '#000000'),
      '#attributes' => [
        'class' => ['wip-color-picker'],
        'data-target-hex' => 'end-hex-input-' . $delta,
      ],
    ];
    $element['gradient_container']['end_color_group']['end_color_wrapper']['wip_end_color_text'] = [
      '#type' => 'textfield',
      '#default_value' => strtoupper($this->formatHexForInput($default_values['wip_end_color'] ?: '#000000')),
      '#size' => 8,
      '#maxlength' => 7,
      '#attributes' => [
        'class' => ['wip-hex-input'],
        'id' => 'end-hex-input-' . $delta,
        'data-target-color-picker' => 'wip-color-picker',
      ],
      '#field_prefix' => '#',
    ];

    $element['gradient_container']['wip_direction'] = [
      '#type' => 'select',
      '#title' => new TranslatableMarkup('Direction'),
      '#description' => new TranslatableMarkup('Enter a value like "180deg", "to right", or "to top left".'),
      '#options' => [
        'to right' => $this->t('To right'),
        'to left' => $this->t('To left'),
        'to bottom' => $this->t('To bottom'),
        'to top' => $this->t('To top'),
        'to bottom right' => $this->t('To bottom right'),
        'to bottom left' => $this->t('To bottom left'),
        'to top right' => $this->t('To top right'),
        'to top left' => $this->t('To top left'),
        '45deg' => $this->t('45 degrees'),
        '90deg' => $this->t('90 degrees'),
        '135deg' => $this->t('135 degrees'),
        '180deg' => $this->t('180 degrees'),
      ],
      '#default_value' => (string) ($default_values['wip_direction']),
      '#states' => [
        'required' => [$selector_path => ['value' => 'gradient']],
      ],
    ];

    unset($element['value']);

    return $element;
  }

  /**
   * Get default values for the field item.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem> $items
   *   The field items.
   * @param int $delta
   *   The delta of the item.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @return array{wip_color_type: string, wip_value: string, wip_end_color: string, wip_direction: string}
   *   Array of default values.
   */
  protected function getDefaultValues(FieldItemListInterface $items, $delta, FormStateInterface $form_state): array {
    $defaults = [
      'wip_color_type' => 'solid',
      'wip_value' => '',
      'wip_end_color' => '',
      'wip_direction' => 'to right',
    ];

    // If we have existing data, use it.
    if (!empty($items[$delta])) {
      $item = $items[$delta];
      $defaults['wip_color_type'] = $item->wip_color_type ?? 'solid';
      $defaults['wip_value'] = $item->wip_value ?? '';
      $defaults['wip_end_color'] = $item->wip_end_color ?? '';
      $defaults['wip_direction'] = $item->wip_direction ?? 'to right';
    }
    // If this is a new item and we're not in the default value widget.
    elseif (!$this->isDefaultValueWidget($form_state)) {
      $field_default = $this->fieldDefinition->getDefaultValueLiteral();
      if (!empty($field_default[0])) {
        $defaults['wip_color_type'] = $field_default[0]['wip_color_type'] ?? 'solid';
        $defaults['wip_value'] = $field_default[0]['wip_value'] ?? '';
        $defaults['wip_end_color'] = $field_default[0]['wip_end_color'] ?? '';
        $defaults['wip_direction'] = $field_default[0]['wip_direction'] ?? 'to right';
      }
    }

    return $defaults;
  }

  /**
   * {@inheritdoc}
   *
   * @param array<int, array<string, mixed>> $values
   *   The array of form values to massage.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<int, array<string, mixed>>
   *   The massaged array of form values.
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    $new_values = [];
    foreach ($values as $value) {
      /** @var array<string, mixed> $value */
      $wip_color_type = $value['wip_color_type'] ?? 'solid';
      $processed_value = ['wip_color_type' => $wip_color_type];

      if ($wip_color_type === 'solid') {
        $solid_container = (array) ($value['solid_container'] ?? []);
        $solid_color_group = (array) ($solid_container['solid_color_group'] ?? []);
        $color_wrapper = (array) ($solid_color_group['color_wrapper'] ?? []);

        $hex_input = $color_wrapper['wip_value_text'] ?? NULL;
        $color_picker = $color_wrapper['wip_value'] ?? NULL;

        $wip_color = $this->selectColorValue(
            is_string($hex_input) ? $hex_input : NULL,
            is_string($color_picker) ? $color_picker : NULL
        );
        $processed_value['wip_value'] = $this->normalizeHex($wip_color);
      }
      elseif ($wip_color_type === 'gradient') {
        $gradient_container = (array) ($value['gradient_container'] ?? []);

        // Star color.
        $start_color_group = (array) ($gradient_container['start_color_group'] ?? []);
        $start_color_wrapper = (array) ($start_color_group['start_color_wrapper'] ?? []);
        $start_hex_input = $start_color_wrapper['wip_value_text'] ?? NULL;
        $start_color_picker = $start_color_wrapper['wip_value'] ?? NULL;
        $start_wip_color = $this->selectColorValue(
            is_string($start_hex_input) ? $start_hex_input : NULL,
            is_string($start_color_picker) ? $start_color_picker : NULL
        );

        // End color.
        $end_color_wrapper = (array) ($gradient_container['end_color_group']['end_color_wrapper'] ?? []);
        $end_hex_input = $end_color_wrapper['wip_end_color_text'] ?? NULL;
        $end_color_picker = $end_color_wrapper['wip_end_color'] ?? NULL;
        $wip_end_color = $this->selectColorValue((string) ($end_hex_input ?? NULL), (string) ($end_color_picker ?? NULL));

        $processed_value['wip_value'] = $this->normalizeHex($start_wip_color);
        $processed_value['wip_end_color'] = $this->normalizeHex($wip_end_color);
        $processed_value['wip_direction'] = $gradient_container['wip_direction'] ?? NULL;
      }
      $new_values[] = $processed_value;
    }
    return $new_values;
  }

  /**
   * Normalizes a hexadecimal color string to the format #RRGGBBAA.
   *
   * This method ensures that the input color string:
   * - Starts with a '#' character.
   * - Is converted to uppercase.
   * - If the color is in the #RRGGBB format (7 characters)
   *   appends 'FF' for full opacity.
   * - Returns the color in #RRGGBBAA format (9 characters) or NULL if invalid.
   *
   * @param string|null $color
   *   The input color string, which may be NULL or in various formats.
   *
   * @return string|null
   *   The normalized color string in #RRGGBBAA format,
   *   or NULL if the input is invalid.
   */
  private function normalizeHex(?string $color): ?string {
    if (empty($color)) {
      return NULL;
    }
    if (substr($color, 0, 1) !== '#') {
      $color = '#' . $color;
    }
    $color = strtoupper($color);
    // If it is a 7-character RGB colour, add FF for full opacity.
    if (strlen($color) === 7) {
      $color .= 'FF';
    }
    return (strlen($color) === 9) ? $color : NULL;
  }

  /**
   * Formats a 9-character hex color string (with alpha) for input fields.
   *
   * If the provided hex color string is 9 characters long and starts with '#',
   * it trims the string to the first 7 characters (removing the alpha channel).
   * Otherwise, it returns the original hex color or an empty string if null.
   *
   * @param string|null $hex_color
   *   The hex color string, possibly including
   *   an alpha channel (e.g., '#RRGGBBAA').
   *
   * @return string
   *   The formatted hex color string suitable for input fields
   *   (e.g., '#RRGGBB'), or an empty string if the input is null.
   */
  protected function formatHexForInput(?string $hex_color): string {
    if ($hex_color && strlen($hex_color) === 9 && substr($hex_color, 0, 1) === '#') {
      $hex_color = substr($hex_color, 0, 7);
    }
    // Remove leading '#' if present.
    if ($hex_color && substr($hex_color, 0, 1) === '#') {
      $hex_color = substr($hex_color, 1);
    }
    return $hex_color ?? '';
  }

  /**
   * Return a valid color value from the provided hex or picker color.
   *
   * @param string|null $hexColor
   *   The hex color string to validate and select, or NULL.
   * @param string|null $pickerColor
   *   The color value selected from a color picker, or NULL.
   *
   * @return string|null
   *   The selected valid color value, or NULL if none is valid.
   */
  private function selectColorValue(?string $hexColor, ?string $pickerColor) : ?string {
    $hexColorWithHash = $hexColor;
    if (!empty($hexColor) && substr($hexColor, 0, 1) !== '#') {
      $hexColorWithHash = '#' . $hexColor;
    }

    if (!empty($hexColorWithHash) && $this->isValidHex($hexColorWithHash)) {
      return $hexColorWithHash;
    }

    return is_string($pickerColor) && $this->isValidHex($pickerColor) ? $pickerColor : NULL;
  }

  /**
   * Validates whether a given string is a valid hexadecimal color code.
   *
   * @param string|null $hex
   *   The color code string to validate.
   *
   * @return bool
   *   TRUE if the string is a valid HEX color code, FALSE otherwise.
   */
  private function isValidHex(?string $hex): bool {
    if (empty($hex)) {
      return FALSE;
    }

    if (substr($hex, 0, 1) !== '#') {
      $hex = '#' . $hex;
    }

    return preg_match('/^#([A-Fa-f0-9]{6}|[A-Fa-f0-9]{3})$/', $hex) === 1;
  }

}
