<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines the 'wip_range_widget' field widget.
 *
 * @FieldWidget(
 * id = "wip_range_widget",
 * label = @Translation("Wip Range Slider"),
 * field_types = {"wip_range"},
 * )
 */
final class WipRangeWidget extends WidgetBase {

  /**
   * {@inheritdoc}
   *
   * @return array<string, mixed>
   *   Default settings for the widget.
   */
  public static function defaultSettings(): array {
    return [
      'display_value' => TRUE,
      'unit_position' => 'after',
      'show_min_max' => TRUE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   *
   * @param array<string, mixed> $form
   *   Form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state object.
   *
   * @return array<string, mixed>
   *   Form elements for the widget settings.
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $element = parent::settingsForm($form, $form_state);
    $settings = $this->getSettings();

    $element['display_value'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display current value next to slider'),
      '#default_value' => $settings['display_value'],
      '#description' => $this->t('If checked, a numerical display will show the current slider value with its unit dynamically.'),
    ];

    $element['unit_position'] = [
      '#type' => 'radios',
      '#title' => $this->t('Unit selector position'),
      '#options' => [
        'before' => $this->t('Before slider'),
        'after' => $this->t('After slider'),
      ],
      '#default_value' => $settings['unit_position'],
      '#description' => $this->t('Position of the unit selector relative to the slider.'),
    ];

    $element['show_min_max'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show min/max values'),
      '#default_value' => $settings['show_min_max'],
      '#description' => $this->t('Display minimum and maximum values around the slider.'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   *
   * @return array<int, \Drupal\Core\StringTranslation\TranslatableMarkup>
   *   Summary of the widget settings.
   */
  public function settingsSummary(): array {
    $summary = [];
    $settings = $this->getSettings();

    $summary[] = $settings['display_value']
        ? $this->t('Display value: Yes')
        : $this->t('Display value: No');

    $summary[] = $this->t(
          'Unit position: @position', [
            '@position' => ucfirst($settings['unit_position']),
          ]
      );

    $summary[] = $settings['show_min_max']
        ? $this->t('Show min/max: Yes')
        : $this->t('Show min/max: No');

    return $summary;
  }

  /**
   * {@inheritdoc}
   *
   * Builds the form element for a single field item.
   * This method renders an HTML5 <input type="color"> element.
   *
   * @param \Drupal\Core\Field\FieldItemListInterface<\Drupal\web_initial_paragraphs\Plugin\Field\FieldType\WipColorItem> $items
   *   The field items being rendered.
   * @param int $delta
   *   The order of this item in the array of field items.
   * @param array<string, mixed> $element
   *   A form element array to populate with the widget's properties.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<string, mixed>
   *   The form element array for the widget.
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $item = $items[$delta] ?? NULL;
    $field_settings = $this->getFieldSettings();
    $widget_settings = $this->getSettings();

    $current_value = ($item !== NULL) ? $item->get('value')->getValue() : ($field_settings['min'] ?? 0);
    $current_unit = ($item !== NULL) ? $item->get('unit')->getValue() : ($field_settings['default_unit'] ?? 'px');

    $allowed_units = array_filter($field_settings['allowed_units'] ?? ['px']);
    $unit_labels = [
      'px' => 'px (Pixels)',
      'rem' => 'rem (Root em)',
      'em' => 'em (Relative to font size)',
      '%' => '% (Percentage)',
      'vw' => 'vw (Viewport width)',
      'vh' => 'vh (Viewport height)',
      'pt' => 'pt (Points)',
      'pc' => 'pc (Picas)',
    ];

    $unit_options = [];
    foreach ($allowed_units as $unit) {
      if (!empty($unit_labels[$unit])) {
        $unit_options[$unit] = $unit_labels[$unit];
      }
    }
    $base_id = $this->getUniqueIdFromElement($element, $delta);
    $range_id = $base_id . '-range';
    $unit_id = $base_id . '-unit';
    $display_id = $base_id . '-display';

    $element += [
      '#type' => 'fieldset',
      '#attributes' => [
        'class' => ['wip-range-widget-wrapper'],
        'data-wip-range-widget-id' => $base_id,
      ],
      '#title' => $this->fieldDefinition->getLabel(),
      '#title_display' => 'before',
    ];

    $unit_element = [
      '#type' => 'select',
      '#title' => $this->t('Unit'),
      '#title_display' => 'invisible',
      '#options' => $unit_options,
      '#default_value' => $current_unit,
      '#attributes' => [
        'id' => $unit_id,
        'class' => ['wip-range-unit-selector'],
        'data-range-target' => $range_id,
      ],
    ];

    $unit_config = $this->getUnitConfig($current_unit, $field_settings);

    $range_element = [
      '#type' => 'range',
      '#title' => $this->t('Value'),
      '#title_display' => 'invisible',
      '#default_value' => $current_value,
      '#min' => $unit_config['min'],
      '#max' => $unit_config['max'],
      '#step' => $unit_config['step'],
      '#attributes' => [
        'id' => $range_id,
        'class' => ['wip-range-slider'],
        'data-unit-target' => $unit_id,
      ],
    ];

    // Layout based on unit position.
    if ($widget_settings['unit_position'] === 'before') {
      $element['unit'] = $unit_element;
      $element['range_container'] = $this->buildRangeContainer($range_element, $unit_config, $widget_settings);
    }
    else {
      $element['range_container'] = $this->buildRangeContainer($range_element, $unit_config, $widget_settings);
      $element['unit'] = $unit_element;
    }

    if ($widget_settings['display_value']) {

      $element['display'] = [
        '#type' => 'html_tag',
        '#tag' => 'div',
        '#value' => $current_value . $current_unit,
        '#attributes' => [
          'id' => $display_id,
          'class' => ['wip-range-value-display'],
          'aria-live' => 'polite',
        ],
      ];
    }

    $element['value'] = [
      '#type' => 'hidden',
      '#default_value' => $current_value,
      '#attributes' => [
        'data-wip-range-value' => $base_id,
      ],
    ];

    $element['unit_hidden'] = [
      '#type' => 'hidden',
      '#default_value' => $current_unit,
      '#attributes' => [
        'data-wip-range-unit' => $base_id,
      ],
    ];

    $element['#attached'] = is_array($element['#attached'] ?? NULL) ? $element['#attached'] : [];
    $element['#attached']['library'] = is_array($element['#attached']['library'] ?? NULL)
      ? $element['#attached']['library']
      : [];
    $element['#attached']['library'][] = 'web_initial_paragraphs/wip-range-widget';

    $element['#attached']['drupalSettings']['wipRangeWidget']['instances'][$base_id] = [
      'rangeId' => $range_id,
      'unitId' => $unit_id,
      'displayId' => $widget_settings['display_value'] ? $display_id : NULL,
      'unitRanges' => $field_settings['unit_ranges'] ?? [],
      'fallbackConfig' => [
        'min' => $field_settings['min'] ?? 0,
        'max' => $field_settings['max'] ?? 100,
        'step' => $field_settings['step'] ?? 1,
      ],
    ];

    return $element;
  }

  /**
   * Builds a render array for a range slider container.
   *
   * This includes optional min/max displays.
   *
   * @param array<string, mixed> $range_element
   *   The render array for the range input element.
   * @param array<string, mixed> $unit_config
   *   An associative array containing unit configuration.
   * @param array<string, mixed> $widget_settings
   *   An associative array of widget settings.
   *
   * @return array<string, mixed>
   *   A render array representing the range slider
   *    container with optional min/max value displays.
   */
  private function buildRangeContainer(array $range_element, array $unit_config, array $widget_settings): array {

    $range_attributes = (array) ($range_element['#attributes'] ?? []);
    $range_id = is_string($range_attributes['id']) ? $range_attributes['id'] : '';

    $container = [
      '#type' => 'container',
      '#attributes' => ['class' => ['wip-range-slider-container']],
    ];

    if ($widget_settings['show_min_max']) {
      $container['min_display'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => is_string($unit_config['min']) ? $unit_config['min'] : '',
        '#attributes' => [
          'class' => ['wip-range-min-value'],
          'data-min-display' => $range_id,
        ],
      ];
    }

    $container['range'] = $range_element;

    if ($widget_settings['show_min_max']) {
      $container['max_display'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => is_string($unit_config['max']) ? $unit_config['max'] : '',
        '#attributes' => [
          'class' => ['wip-range-max-value'],
          'data-max-display' => $range_id,
        ],
      ];
    }

    return $container;
  }

  /**
   * Get unit configuration.
   *
   * @param string $unit
   *   The unit identifier for which to retrieve configuration.
   * @param array<string, mixed> $field_settings
   *   The field settings array, potentially containing
   *   'unit_ranges' and default values.
   *
   * @return array<string, float>
   *   An associative array with the following keys:
   */
  private function getUnitConfig(string $unit, array $field_settings): array {
    $unit_ranges = (array) ($field_settings['unit_ranges'] ?? []);
    $unit_config = (array) ($unit_ranges[$unit] ?? []);

    return [
      'min' => is_numeric($unit_config['min'] ?? NULL) ? (float) $unit_config['min'] : (is_numeric($field_settings['min'] ?? NULL) ? (float) $field_settings['min'] : 0),
      'max' => (float) ($unit_config['max'] ?? $field_settings['max'] ?? 100),
      'step' => (float) ($unit_config['step'] ?? $field_settings['step'] ?? 1),
    ];
  }

  /**
   * {@inheritdoc}
   *
   * @param array<int, array<string, mixed>> $values
   *   The array of form values to massage.
   * @param array<string, mixed> $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array<int, array<string, mixed>>
   *   The massaged array of form values.
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state): array {
    foreach ($values as &$value) {
      // Extract values from the form.
      if (isset($value['value']) && $value['value'] !== '') {
        $value['value'] = is_numeric($value['value']) ? (float) $value['value'] : NULL;
      }
      else {
        $value['value'] = NULL;
      }

      if (isset($value['unit_hidden'])) {
        $value['unit'] = $value['unit_hidden'];
        unset($value['unit_hidden']);
      }

      // Clean up unnecessary form elements.
      unset($value['range_container'], $value['display']);
    }
    return $values;
  }

  /**
   * Helper to get a unique HTML-safe ID for the field instance.
   *
   * @param array<string, mixed> $element
   *   The form element.
   * @param int $delta
   *   The field delta.
   *
   * @return string
   *   Unique HTML-safe identifier.
   */
  private function getUniqueIdFromElement(array $element, int $delta): string {
    $field_name = $this->fieldDefinition->getName();
    $parents = is_array($element['#array_parents'] ?? NULL) ? $element['#array_parents'] : [];

    $id_parts = array_merge($parents, [$field_name, $delta]);
    $clean_parts = array_map(
          function ($part) {
              return preg_replace('/[^a-zA-Z0-9]+/', '-', is_string($part) ? (string) $part : '');
          }, $id_parts
      );

    return 'wip-range-' . implode('-', array_filter($clean_parts));
  }

}
