<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Service;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Field\FormatterPluginManager;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Psr\Log\LoggerInterface;

/**
 * Service for mapping WIP field types to template variables.
 */
final class WipFieldMapper implements WipFieldMapperInterface {

  use StringTranslationTrait;

  /**
   * Cache of instantiated formatters to avoid recreation.
   *
   * @var array<string, mixed>
   */
  private array $formatterCache = [];

  /**
   * Cache of field type mappings per entity type/bundle.
   *
   * @var array<string, mixed>
   */
  private array $fieldMappingCache = [];

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Constructs a WipFieldMapper service.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cacheTagsInvalidator
   *   The cache tags invalidator.
   * @param \Drupal\Core\Entity\EntityDisplayRepositoryInterface $entityDisplayRepository
   *   The entity display repository.
   * @param \Drupal\Core\Field\FormatterPluginManager $formatterPluginManager
   *   The field formatter plugin manager.
   * @param \Drupal\Core\Language\LanguageManagerInterface $languageManager
   *   The language manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $stringTranslation
   *   The string translation service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger channel factory.
   */
  public function __construct(
    private readonly CacheBackendInterface $cache,
    private readonly CacheTagsInvalidatorInterface $cacheTagsInvalidator,
    private readonly EntityDisplayRepositoryInterface $entityDisplayRepository,
    private readonly FormatterPluginManager $formatterPluginManager,
    private readonly LanguageManagerInterface $languageManager,
    private readonly RendererInterface $renderer,
    TranslationInterface $stringTranslation,
    LoggerChannelFactoryInterface $loggerFactory,
  ) {
    $this->setStringTranslation($stringTranslation);
    $this->logger = $loggerFactory->get('web_initial_paragraphs');
  }

  /**
   * Maps supported fields of a content entity to template variables.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity from which to map fields.
   * @param string $view_mode
   *   (optional) The view mode to use for rendering field output.
   *     Defaults to 'default'.
   * @param array<string> $supported_field_types
   *   (optional) An array of field type machine names to include.
   *     Defaults to ['wip_color', 'wip_range'].
   *
   * @return array<string, mixed>
   *   An associative array where the keys are variable names
   *    derived from field names, and the values are the rendered field outputs.
   */
  public function mapFieldsToVariables(
    ContentEntityInterface $entity,
    string $view_mode = 'default',
    array $supported_field_types = ['wip_color', 'wip_range'],
  ): array {
    $cache_id = $this->buildCacheId($entity, $view_mode, $supported_field_types);

    $cached = $this->cache->get($cache_id);
    if ($cached !== FALSE) {
      /** @var array<string, mixed> $cached_data */
      $cached_data = $cached->data;
      return $cached_data;
    }

    $variables = [];
    $fields = $this->getFieldsByType($entity, $supported_field_types);

    foreach ($fields as $field_name => $field_type) {
      $rendered_output = $this->renderFieldWithFormatter($entity, $field_name, $view_mode);

      if (!empty($rendered_output)) {
        $variable_name = $this->fieldNameToVariableName($field_name);
        $variables[$variable_name] = $rendered_output;
      }
    }

    $cache_tags = $this->buildCacheTags($entity, array_keys($fields));
    $this->cache->set($cache_id, $variables, CacheBackendInterface::CACHE_PERMANENT, $cache_tags);

    return $variables;
  }

  /**
   * {@inheritdoc}
   */

  /**
   * Retrieves an array of field names and their types for the given entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity from which to retrieve field definitions.
   * @param array<string> $field_types
   *   An array of field type machine names to filter by.
   *
   * @return array<string, mixed>
   *   An associative array where the keys are field names and the values are
   *   the corresponding field type machine names.
   */
  public function getFieldsByType(ContentEntityInterface $entity, array $field_types): array {
    $entity_type = $entity->getEntityTypeId();
    $bundle = $entity->bundle();
    $cache_key = "{$entity_type}:{$bundle}:" . implode(',', $field_types);

    if (isset($this->fieldMappingCache[$cache_key])) {
      /** @var array<string, string> $cached_fields */
      $cached_fields = $this->fieldMappingCache[$cache_key];
      return $cached_fields;
    }

    $fields = [];
    $field_definitions = $entity->getFieldDefinitions();

    foreach ($field_definitions as $field_name => $field_definition) {
      $field_type = $field_definition->getType();

      if (in_array($field_type, $field_types, TRUE)) {
        $fields[$field_name] = $field_type;
      }
    }

    $this->fieldMappingCache[$cache_key] = $fields;

    return $fields;
  }

  /**
   * Renders a field of a content entity using the specified formatter.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity containing the field to render.
   * @param string $field_name
   *   The machine name of the field to render.
   * @param string $view_mode
   *   (optional) The view mode to use for rendering. Defaults to 'default'.
   *
   * @return string
   *   The rendered field output as a string, or an empty
   *   string if the field cannot be rendered.
   */
  public function renderFieldWithFormatter(
    ContentEntityInterface $entity,
    string $field_name,
    string $view_mode = 'default',
  ): string {
    if (!$entity->hasField($field_name) || $entity->get($field_name)->isEmpty()) {
      return '';
    }

    $field_items = $entity->get($field_name);
    $display = $this->entityDisplayRepository->getViewDisplay(
          $entity->getEntityTypeId(),
          $entity->bundle(),
          $view_mode
      );

    $component = $display->getComponent($field_name);
    if (!$component) {
      return '';
    }

    $formatter = $this->getFormatter($field_items->getFieldDefinition(), $component, $view_mode);
    if ($formatter === NULL) {
      return '';
    }
    $langcode = $this->languageManager->getCurrentLanguage()->getId();
    $build = $formatter->viewElements($field_items, $langcode);

    if (empty($build)) {
      return '';
    }

    return (string) $this->renderer->renderInIsolation($build);
  }

  /**
   * Converts a field name to a variable name by removing the 'field_' prefix.
   *
   * @param string $field_name
   *   The original field name, typically prefixed with 'field_'.
   *
   * @return string
   *   The variable name with the 'field_' prefix removed.
   */
  public function fieldNameToVariableName(string $field_name): string {
    $variable_name = preg_replace('/^field_/', '', $field_name);

    return (string) $variable_name;
  }

  /**
   * Clears cache entries and invalidates cache tags for a given content entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The content entity for which to clear the cache.
   */
  public function clearEntityCache(ContentEntityInterface $entity): void {
    $entity_type = $entity->getEntityTypeId();
    $entity_id = $entity->id();

    $supported_field_types = ['wip_color', 'wip_range'];
    $view_modes = ['default', 'teaser', 'full'];

    foreach ($view_modes as $view_mode) {
      $cache_id = $this->buildCacheId($entity, $view_mode, $supported_field_types);
      $this->cache->delete($cache_id);
    }

    $bundle = $entity->bundle();
    $cache_key = "{$entity_type}:{$bundle}:" . implode(',', $supported_field_types);
    unset($this->fieldMappingCache[$cache_key]);

    $cache_tags = [
      "{$entity_type}:{$entity_id}",
      'field_config_list',
    ];

    $this->cacheTagsInvalidator->invalidateTags($cache_tags);
  }

  /**
   * Clears cached field mappings and formatters for a specific entity bundle.
   *
   * @param string $entity_type
   *   The machine name of the entity type (e.g., 'node', 'user').
   * @param string $bundle
   *   The machine name of the bundle (e.g., 'article', 'page').
   */
  public function clearBundleCache(string $entity_type, string $bundle): void {
    $supported_field_types = ['wip_color', 'wip_range'];

    // Clear field mapping cache for this bundle.
    $cache_key = "{$entity_type}:{$bundle}:" . implode(',', $supported_field_types);
    unset($this->fieldMappingCache[$cache_key]);

    // Clear formatter cache for this bundle.
    $pattern = preg_quote("{$entity_type}:{$bundle}:", '/') . '.*';
    $this->formatterCache = array_filter(
        $this->formatterCache,
        function ($key) use ($pattern) {
            return !preg_match("/^{$pattern}$/", $key);
        },
        ARRAY_FILTER_USE_KEY
    );

    // Invalidate cache tags for this bundle.
    $this->cacheTagsInvalidator->invalidateTags(
          [
            "field_config:{$entity_type}:{$bundle}",
            "{$entity_type}_type:{$bundle}",
          ]
      );
  }

  /**
   * Gets or creates a formatter instance.
   *
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The field definition.
   * @param array<string, mixed> $component
   *   The display component configuration.
   * @param string $view_mode
   *   The view mode.
   *
   * @return \Drupal\Core\Field\FormatterInterface|null
   *   The formatter instance or null if creation failed.
   */
  private function getFormatter($field_definition, array $component, string $view_mode) {
    $formatter_id = (string) $component['type'];
    $formatter_settings = (array) $component['settings'];

    // Create a cache key for this formatter configuration.
    $cache_key = implode(
          ':', [
            $field_definition->getTargetEntityTypeId(),
            $field_definition->getTargetBundle(),
            $field_definition->getName(),
            $formatter_id,
            md5(serialize($formatter_settings)),
            $view_mode,
          ]
      );

    // Return cached formatter if available.
    if (isset($this->formatterCache[$cache_key])) {
      $cached_formatter = $this->formatterCache[$cache_key];
      return $cached_formatter;
    }

    try {
      /** @var \Drupal\Core\Field\FormatterInterface $formatter */
      $formatter = $this->formatterPluginManager->createInstance(
            $formatter_id, [
              'field_definition' => $field_definition,
              'settings' => $formatter_settings,
              'label' => $component['label'] ?? 'hidden',
              'view_mode' => $view_mode,
              'third_party_settings' => $component['third_party_settings'] ?? [],
            ]
        );

      // Cache the formatter instance.
      $this->formatterCache[$cache_key] = $formatter;

      return $formatter;
    }
    catch (\Exception $e) {
      // Log the error but don't break the page.
      $this->logger->error(
            'Failed to create formatter @formatter_id for field @field_name: @message',
            [
              '@formatter_id' => $formatter_id,
              '@field_name' => $field_definition->getName(),
              '@message' => $e->getMessage(),
            ]
        );

      return NULL;
    }
  }

  /**
   * Builds a cache ID for the given parameters.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param string $view_mode
   *   The view mode.
   * @param array $field_types
   *   The field types to process.
   *
   * @return string
   *   The cache ID.
   */
  private function buildCacheId(ContentEntityInterface $entity, string $view_mode, array $field_types): string {
    return implode(
          ':', [
            'wip_field_mapper',
            $entity->getEntityTypeId(),
            $entity->bundle(),
            $entity->id(),
            $view_mode,
            implode(',', $field_types),
          ]
      );
  }

  /**
   * Builds cache tags for the given entity and fields.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param array $field_names
   *   The field names that were processed.
   *
   * @return array
   *   Array of cache tags.
   */
  private function buildCacheTags(ContentEntityInterface $entity, array $field_names): array {
    $cache_tags = [
      $entity->getEntityTypeId() . ':' . $entity->id(),
      'field_config_list',
    ];

    // Add field-specific cache tags.
    foreach ($field_names as $field_name) {
      $cache_tags[] = "field_config:{$entity->getEntityTypeId()}:{$entity->bundle()}:{$field_name}";
    }

    return $cache_tags;
  }

}
