<?php

declare(strict_types=1);

namespace Drupal\web_initial_paragraphs\Service;

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Interface for mapping WIP field types to template variables.
 */
interface WipFieldMapperInterface {

  /**
   * Maps WIP fields to template variables for a given entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity containing the fields to map.
   * @param string $view_mode
   *   The view mode being used to display the entity.
   * @param array $supported_field_types
   *   Array of supported field types to process.
   *   Defaults to ['wip_color', 'wip_range'].
   *
   * @return array
   *   An associative array of template variables where:
   *   - Key: derived from field name
   *     (e.g., 'field_wip_cta_bg_color' becomes 'wip_cta_bg_color')
   *   - Value: the rendered output from the field's formatter
   */
  public function mapFieldsToVariables(
    ContentEntityInterface $entity,
    string $view_mode = 'default',
    array $supported_field_types = ['wip_color', 'wip_range'],
  ): array;

  /**
   * Gets all fields of specified types from an entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to scan for fields.
   * @param array $field_types
   *   Array of field types to search for.
   *
   * @return array
   *   Array of field names keyed by field name.
   */
  public function getFieldsByType(ContentEntityInterface $entity, array $field_types): array;

  /**
   * Renders a field using its configured formatter.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity containing the field.
   * @param string $field_name
   *   The name of the field to render.
   * @param string $view_mode
   *   The view mode being used.
   *
   * @return string
   *   The rendered field output.
   */
  public function renderFieldWithFormatter(
    ContentEntityInterface $entity,
    string $field_name,
    string $view_mode = 'default',
  ): string;

  /**
   * Converts a field name to a template variable name.
   *
   * @param string $field_name
   *   The field name (e.g., 'field_wip_cta_bg_color').
   *
   * @return string
   *   The template variable name (e.g., 'wip_cta_bg_color').
   */
  public function fieldNameToVariableName(string $field_name): string;

  /**
   * Clears the cache for a specific entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity to clear cache for.
   */
  public function clearEntityCache(ContentEntityInterface $entity): void;

  /**
   * Clears cache for all entities of a specific bundle.
   *
   * @param string $entity_type
   *   The entity type ID.
   * @param string $bundle
   *   The bundle ID.
   */
  public function clearBundleCache(string $entity_type, string $bundle): void;

}
